@@ -18,8 +18,9 @@ from newrelic.core.internal_metrics import (internal_trace, internal_metric)
 # another module. Need to investigate a better way of handling that at
 # some point.
 
-def _dbapi_name(dbapi):
-    return dbapi and hasattr(dbapi, '__name__') and dbapi.__name__ or dbapi
+def _dbapi2_module_name(dbapi2_module):
+    return (dbapi2_module and hasattr(dbapi2_module, '__name__') and
+            dbapi2_module.__name__ or dbapi2_module)
 
 # Obfuscation of SQL is done when reporting SQL statements back to the
 # data collector so that sensitive information is not being passed.
@@ -64,10 +65,18 @@ _quotes_table = {
 _quotes_default = _single_quotes_re
 
 @internal_trace('Supportability/DatabaseUtils/Calls/obfuscate_sql')
-def _obfuscate_sql(sql, dbapi=None):
-    name = _dbapi_name(dbapi)
+def _obfuscate_sql(sql, dbapi2_module=None):
+    quoting_style = getattr(dbapi2_module, '_nr_quoting_style', None)
 
-    quotes_re = _quotes_table.get(name, _quotes_default)
+    if quoting_style == 'single':
+        quotes_re = _single_quotes_re
+    elif quoting_style == 'double':
+        quotes_re = _double_quotes_re
+    elif quoting_style == 'single+double':
+        quotes_re = _any_quotes_re
+    else:
+        name = _dbapi2_module_name(dbapi2_module)
+        quotes_re = _quotes_table.get(name, _quotes_default)
 
     # Substitute quoted strings first.
 
@@ -118,7 +127,7 @@ _normalize_whitespace_3_p = r'(?<!\w)\s+'
 _normalize_whitespace_3_re = re.compile(_normalize_whitespace_3_p)
 
 @internal_trace('Supportability/DatabaseUtils/Calls/normalize_sql')
-def _normalize_sql(sql, dbapi):
+def _normalize_sql(sql, dbapi2_module):
     # Note we that do this as a series of regular expressions as
     # using '|' in regular expressions is more expensive.
 
@@ -170,7 +179,7 @@ def _extract_identifier(token):
 _uncomment_sql_p = r'/\*.*?\*/'
 _uncomment_sql_re = re.compile(_uncomment_sql_p, re.DOTALL)
 
-def _uncomment_sql(sql, dbapi):
+def _uncomment_sql(sql, dbapi2_module):
     return _uncomment_sql_re.sub('', sql)
 
 # Parser routines for the different SQL statement operation types.
@@ -238,12 +247,12 @@ _parse_from_p = '\s+FROM\s+' + _parse_identifier_p
 _parse_from_re = re.compile(_parse_from_p, re.IGNORECASE)
 
 @internal_trace('Supportability/DatabaseUtils/Calls/parse_target_select')
-def _parse_select(sql, dbapi):
+def _parse_select(sql, dbapi2_module):
     m = _parse_from_re.search(sql)
     return m and next(s for s in m.groups()[1:] if s).lower() or ''
 
 @internal_trace('Supportability/DatabaseUtils/Calls/parse_target_delete')
-def _parse_delete(sql, dbapi):
+def _parse_delete(sql, dbapi2_module):
     m = _parse_from_re.search(sql)
     return m and next(s for s in m.groups()[1:] if s).lower() or ''
 
@@ -251,7 +260,7 @@ _parse_into_p = '\s+INTO\s+' + _parse_identifier_p
 _parse_into_re = re.compile(_parse_into_p, re.IGNORECASE)
 
 @internal_trace('Supportability/DatabaseUtils/Calls/parse_target_insert')
-def _parse_insert(sql, dbapi):
+def _parse_insert(sql, dbapi2_module):
     m = _parse_into_re.search(sql)
     return m and next(s for s in m.groups()[1:] if s).lower() or ''
 
@@ -259,7 +268,7 @@ _parse_update_p = '\s*UPDATE\s+' + _parse_identifier_p
 _parse_update_re = re.compile(_parse_update_p, re.IGNORECASE)
 
 @internal_trace('Supportability/DatabaseUtils/Calls/parse_target_update')
-def _parse_update(sql, dbapi):
+def _parse_update(sql, dbapi2_module):
     m = _parse_update_re.search(sql)
     return m and next(s for s in m.groups()[1:] if s).lower() or ''
 
@@ -267,12 +276,12 @@ _parse_table_p = '\s+TABLE\s+' + _parse_identifier_p
 _parse_table_re = re.compile(_parse_table_p, re.IGNORECASE)
 
 @internal_trace('Supportability/DatabaseUtils/Calls/parse_target_create')
-def _parse_create(sql, dbapi):
+def _parse_create(sql, dbapi2_module):
     m = _parse_table_re.search(sql)
     return m and next(s for s in m.groups()[1:] if s).lower() or ''
 
 @internal_trace('Supportability/DatabaseUtils/Calls/parse_target_drop')
-def _parse_drop(sql, dbapi):
+def _parse_drop(sql, dbapi2_module):
     m = _parse_table_re.search(sql)
     return m and next(s for s in m.groups()[1:] if s).lower() or ''
 
@@ -283,42 +292,42 @@ _parse_call_p = r'\s*CALL\s+(?!\()(\w+)'
 _parse_call_re = re.compile(_parse_call_p, re.IGNORECASE)
 
 @internal_trace('Supportability/DatabaseUtils/Calls/parse_target_call')
-def _parse_call(sql, dbapi):
+def _parse_call(sql, dbapi2_module):
     return _parse_default(sql, _parse_call_re)
 
 _parse_show_p = r'\s*SHOW\s+(.*)'
 _parse_show_re = re.compile(_parse_show_p, re.IGNORECASE | re.DOTALL)
 
 @internal_trace('Supportability/DatabaseUtils/Calls/parse_target_show')
-def _parse_show(sql, dbapi):
+def _parse_show(sql, dbapi2_module):
     return _parse_default(sql, _parse_show_re)
 
 _parse_set_p = r'\s*SET\s+(.*?)\W+.*'
 _parse_set_re = re.compile(_parse_set_p, re.IGNORECASE | re.DOTALL)
 
 @internal_trace('Supportability/DatabaseUtils/Calls/parse_target_set')
-def _parse_set(sql, dbapi):
+def _parse_set(sql, dbapi2_module):
     return _parse_default(sql, _parse_set_re)
 
 _parse_exec_p = r'\s*EXEC\s+(?!\()(\w+)'
 _parse_exec_re = re.compile(_parse_exec_p, re.IGNORECASE)
 
 @internal_trace('Supportability/DatabaseUtils/Calls/parse_target_exec')
-def _parse_exec(sql, dbapi):
+def _parse_exec(sql, dbapi2_module):
     return _parse_default(sql, _parse_exec_re)
 
 _parse_execute_p = r'\s*EXECUTE\s+(?!\()(\w+)'
 _parse_execute_re = re.compile(_parse_execute_p, re.IGNORECASE)
 
 @internal_trace('Supportability/DatabaseUtils/Calls/parse_target_execute')
-def _parse_execute(sql, dbapi):
+def _parse_execute(sql, dbapi2_module):
     return _parse_default(sql, _parse_execute_re)
 
 _parse_alter_p = r'\s*ALTER\s+(?!\()(\w+)'
 _parse_alter_re = re.compile(_parse_alter_p, re.IGNORECASE)
 
 @internal_trace('Supportability/DatabaseUtils/Calls/parse_target_alter')
-def _parse_alter(sql, dbapi):
+def _parse_alter(sql, dbapi2_module):
     return _parse_default(sql, _parse_alter_re)
 
 _operation_table = {
@@ -340,15 +349,15 @@ _parse_operation_p = r'(\w+)'
 _parse_operation_re = re.compile(_parse_operation_p)
 
 @internal_trace('Supportability/DatabaseUtils/Calls/parse_operation')
-def _parse_operation(sql, dbapi):
+def _parse_operation(sql, dbapi2_module):
     match = _parse_operation_re.search(sql)
     operation = match and match.group(1).lower() or ''
     return operation if operation in _operation_table else ''
 
 @internal_trace('Supportability/DatabaseUtils/Calls/parse_target')
-def _parse_target(sql, dbapi, operation):
+def _parse_target(sql, dbapi2_module, operation):
     parse = _operation_table.get(operation, None)
-    return parse and parse(sql, dbapi) or ''
+    return parse and parse(sql, dbapi2_module) or ''
 
 _explain_plan_table = {
     'MySQLdb': ('EXPLAIN', ('select',)),
@@ -367,27 +376,28 @@ _explain_plan_table = {
 }
 
 @internal_trace('Supportability/DatabaseUtils/Calls/explain_plan')
-def _explain_plan(sql, dbapi, connect_params, cursor_params, execute_params):
-    if dbapi is None:
-        return None
-    if type(dbapi) == type(''):
+def _explain_plan(sql, dbapi2_module, connect_params, cursor_params,
+        execute_params):
+
+    if dbapi2_module is None:
         return None
 
-    name = _dbapi_name(dbapi)
+    name = _dbapi2_module_name(dbapi2_module)
 
     if name is None:
         return None
 
     if connect_params is None:
         return None
-    if cursor_params is None:
-        return None
-    if execute_params is None:
-        return None
 
     query = None
 
-    command, operations = _explain_plan_table.get(name, (None, None))
+    command = getattr(dbapi2_module, '_nr_explain_query', None)
+
+    if command:
+        operations = dbapi2_module._nr_explain_stmts
+    else:
+        command, operations = _explain_plan_table.get(name, (None, None))
 
     if not command:
         return None
@@ -400,7 +410,7 @@ def _explain_plan(sql, dbapi, connect_params, cursor_params, execute_params):
     # going through the SQL cache here as only doing this for a small
     # number of SQL statements per application per harvest cycle.
 
-    operation = _parse_operation(sql, dbapi)
+    operation = _parse_operation(sql, dbapi2_module)
 
     if operation not in operations:
         return None
@@ -409,13 +419,18 @@ def _explain_plan(sql, dbapi, connect_params, cursor_params, execute_params):
 
     args, kwargs = connect_params
     try:
-        connection = dbapi.connect(*args, **kwargs)
+        connection = dbapi2_module.connect(*args, **kwargs)
         try:
-            args, kwargs = cursor_params
-            cursor = connection.cursor(*args, **kwargs)
+            if cursor_params is not None:
+                args, kwargs = cursor_params
+                cursor = connection.cursor(*args, **kwargs)
+            else:
+                cursor = connection.cursor()
             try:
-                args, kwargs = execute_params
-                cursor.execute(query, *args, **kwargs)
+                if execute_params is not None:
+                    cursor.execute(query, execute_params)
+                else:
+                    cursor.execute(query)
                 columns = []
                 if cursor.description:
                     for column in cursor.description:
@@ -431,7 +446,7 @@ def _explain_plan(sql, dbapi, connect_params, cursor_params, execute_params):
         finally:
             try:
                 connection.rollback()
-            except (AttributeError, dbapi.NotSupportedError):
+            except (AttributeError, dbapi2_module.NotSupportedError):
                 pass
             connection.close()
     except Exception:
@@ -441,9 +456,9 @@ def _explain_plan(sql, dbapi, connect_params, cursor_params, execute_params):
 
 class SQLStatement(object):
 
-    def __init__(self, sql, dbapi=None):
+    def __init__(self, sql, dbapi2_module=None):
         self.sql = sql
-        self.dbapi = dbapi
+        self.dbapi2_module = dbapi2_module
 
         self._operation = None
         self._target = None
@@ -455,32 +470,35 @@ class SQLStatement(object):
     @property
     def operation(self):
         if self._operation is None:
-            self._operation = _parse_operation(self.uncommented, self.dbapi)
+            self._operation = _parse_operation(self.uncommented,
+                    self.dbapi2_module)
         return self._operation
 
     @property
     def target(self):
         if self._target is None:
-            self._target = _parse_target(self.uncommented, self.dbapi,
+            self._target = _parse_target(self.uncommented, self.dbapi2_module,
                     self.operation)
         return self._target
 
     @property
     def uncommented(self):
         if self._uncommented is None:
-            self._uncommented = _uncomment_sql(self.sql, self.dbapi)
+            self._uncommented = _uncomment_sql(self.sql, self.dbapi2_module)
         return self._uncommented
 
     @property
     def obfuscated(self):
         if self._obfuscated is None:
-            self._obfuscated = _obfuscate_sql(self.uncommented, self.dbapi)
+            self._obfuscated = _obfuscate_sql(self.uncommented,
+                    self.dbapi2_module)
         return self._obfuscated
 
     @property
     def normalized(self):
         if self._normalized is None:
-            self._normalized = _normalize_sql(self.obfuscated, self.dbapi)
+            self._normalized = _normalize_sql(self.obfuscated,
+                    self.dbapi2_module)
         return self._normalized
 
     @property
@@ -500,20 +518,20 @@ class SQLStatement(object):
             return self.obfuscated
 
     def explain_plan(self, connect_params, cursor_params, execute_params):
-        return _explain_plan(self.sql, self.dbapi, connect_params,
+        return _explain_plan(self.sql, self.dbapi2_module, connect_params,
                 cursor_params, execute_params)
 
 _sql_statements = weakref.WeakValueDictionary()
 
-def sql_statement(sql, dbapi):
-    key = (sql, dbapi)
+def sql_statement(sql, dbapi2_module):
+    key = (sql, dbapi2_module)
 
     result = _sql_statements.get(key, None)
 
     if result is not None:
         return result
 
-    result = SQLStatement(sql, dbapi)
+    result = SQLStatement(sql, dbapi2_module)
     _sql_statements[key] = result
 
     return result
