@@ -27,13 +27,6 @@ try:
 except ImportError:
     pass
 
-try:
-    import newrelic.packages.simplejson._speedups
-except ImportError:
-    pass
-
-import newrelic.packages.six as six
-
 def environment_settings():
     """Returns an array of arrays of environment settings
 
@@ -81,9 +74,6 @@ def environment_settings():
     if 'newrelic.core._thread_utilization' in sys.modules:
         extensions.append('newrelic.core._thread_utilization')
 
-    if 'newrelic.packages.simplejson._speedups' in sys.modules:
-        extensions.append('newrelic.packages.simplejson._speedups')
-
     env.append(('Compiled Extensions', ', '.join(extensions)))
 
     # Dispatcher information.
@@ -153,7 +143,16 @@ def environment_settings():
 
     plugins = []
 
-    for name, module in list(six.iteritems(sys.modules)):
+    # Using six to create create a snapshot of sys.modules can occassionally
+    # fail in a rare case when modules are imported in parallel by different
+    # threads. This is because list(six.iteritems(sys.modules)) results in
+    # list(iter(sys.modules.iteritems())), which means sys.modules could change
+    # between the time when the iterable is handed over from the iter() to
+    # list().
+    #
+    # TL;DR: Do NOT use six module for the following iteration.
+
+    for name, module in list(sys.modules.items()):
         if name.startswith('newrelic.hooks.'):
             plugins.append(name)
 
